searchData={"content_type":"text/markdown","items":[{"doc":"The `FaktoryWorker` module provides everything required to setup workers for sending and fetching jobs.\n\nIt is expected that FaktoryWorker will be configured and started as part of a supervision tree. Multiple instances of\nFaktoryWorker can be configured by providing the `:name`  option which must be unique.\n\nThis module can either be configured using all default options.\n\n```elixir\nchildren = [\n  FaktoryWorker\n]\n```\n\nOr by using the two element tuple format accepting a list of options as the second element.\n\n```elixir\nchildren = [\n  {FaktoryWorker, [name: :faktory_test, ...]}\n]\n```\n\nFor a full list of configuration options see the [Configuration](configuration.html) documentation.","ref":"FaktoryWorker.html","title":"FaktoryWorker","type":"module"},{"doc":"Attaches the default telemetry handler provided by FaktoryWorker.\n\nThis function attaches the default telemetry handler provided by FaktoryWorker that\noutputs log messages for each of the events emitted by FaktoryWorker.\n\nFor a full list of events see the [Logging](logging.html) documentation.","ref":"FaktoryWorker.html#attach_default_telemetry_handler/0","title":"FaktoryWorker.attach_default_telemetry_handler/0","type":"function"},{"doc":"Send a command to the Faktory server.\n\nIn most cases, `FaktoryWorker` handles sending commands on your behalf.\nHowever, there are some APIs (such as Enterprise Batches and Tracking)\nwhere it's useful to send a command directly.\n\nThe full list of supported commands is available in\n`FaktoryWorker.Protocol.protocol_command()`. It is left to the caller to\nverify that command arguments are valid.","ref":"FaktoryWorker.html#send_command/2","title":"FaktoryWorker.send_command/2","type":"function"},{"doc":"- `faktory_name` the Faktory instance to use (default: FaktoryWorker)\n- `timeout` how long to wait for a response, in ms (default: 5000)","ref":"FaktoryWorker.html#send_command/2-options","title":"Options - FaktoryWorker.send_command/2","type":"function"},{"doc":"","ref":"FaktoryWorker.html#t:command/0","title":"FaktoryWorker.command/0","type":"type"},{"doc":"","ref":"FaktoryWorker.html#t:send_command_opt/0","title":"FaktoryWorker.send_command_opt/0","type":"type"},{"doc":"Supports Faktory Batch operations\n\n[Batch support](https://github.com/contribsys/faktory/wiki/Ent-Batches) is a\nFaktory Enterprise feature. It allows jobs to pushed as part of a batch. When\nall jobs in a batch have completed, Faktory will queue a callback job. This\nallows building complex job workflows with dependencies.","ref":"FaktoryWorker.Batch.html","title":"FaktoryWorker.Batch","type":"module"},{"doc":"A batch is created using `new!/1` and must provide a description and declare\none of the success or complete callbacks. The `new!/1` function returns the\nbatch ID (or `bid`) which identifies the batch for future commands.\n\nOnce created, jobs can be pushed to the batch by providing the `bid` in the\n`custom` payload. These jobs must be pushed synchronously.\n\n```\nalias FaktoryWorker.Batch\n\n{:ok, bid} = Batch.new!(on_success: {MyApp.EmailReportJob, [], []})\nMyApp.Job.perform_async([1, 2], custom: %{\"bid\" => bid})\nMyApp.Job.perform_async([3, 4], custom: %{\"bid\" => bid})\nMyApp.Job.perform_async([5, 6], custom: %{\"bid\" => bid})\nBatch.commit(bid)\n```","ref":"FaktoryWorker.Batch.html#module-creating-a-batch","title":"Creating a batch - FaktoryWorker.Batch","type":"module"},{"doc":"In order to open a batch, you must know the batch ID. Since FaktoryWorker\ndoesn't currently pass the job itself as a parameter to `perform` functions,\nyou must explicitly pass it as an argument in order to open the batch as part\nof a job.\n\n```\ndefmodule MyApp.Job do\n  use FaktoryWorker.Job\n\n  def perform(arg1, arg2, bid) do\n    Batch.open(bid)\n\n    MyApp.OtherJob.perform_async([1, 2], custom: %{\"bid\" => bid})\n\n    Batch.commit(bid)\n  end\nend\n```","ref":"FaktoryWorker.Batch.html#module-opening-a-batch","title":"Opening a batch - FaktoryWorker.Batch","type":"module"},{"doc":"Commits the batch identified by `bid`\n\nFaktory will begin scheduling jobs that are part of the batch before the batch\nis committed, but","ref":"FaktoryWorker.Batch.html#commit/2","title":"FaktoryWorker.Batch.commit/2","type":"function"},{"doc":"Creates a new Faktory batch\n\nReturns the batch ID (`bid`) which needs to be passed in the `:custom`\nparameters of every job that should be part of this batch as well as to commit\nthe batch.","ref":"FaktoryWorker.Batch.html#new!/1","title":"FaktoryWorker.Batch.new!/1","type":"function"},{"doc":"Batch jobs must define a success or complete callback (or both). These\ncallbacks are passed as tuples to the `:on_success` and `:on_complete` opts.\nThey are defined as a tuple consisting of `{mod, args, opts}` where `mod` is a\nmodule with a `perform` function that corresponds in arity to the length of `args`.\n\nAny `opts` that can be passed to `perform_async/2` can be provided as `opts`\nto the callback except for `:faktory_worker`.\n\nIf neither callback is provided, an error will be raised.\n\n### `:on_success`\n\nSee above.\n\n### `:on_complete`\n\nSee above.\n\n### `:description`\n\nThe description, if provided, is shown in Faktory's Web UI on the batch\nlisting tab.\n\n### `:parent_bid`\n\nThe parent batch ID--only used if you are creating a child batch.\n\n### `:faktory_name`\n\nThe name of the `FaktoryWorker` instance (determines which connection pool\nwill be used).\n\n### `:timeout`\n\nHow long to wait for a response, in ms.","ref":"FaktoryWorker.Batch.html#new!/1-opts","title":"Opts - FaktoryWorker.Batch.new!/1","type":"function"},{"doc":"Opens the batch identified by `bid`\n\nAn existing batch needs to be re-opened in order to add more jobs to it or to\nadd a child batch.\n\nAfter opening the batch, it must be committed again using `commit/2`.","ref":"FaktoryWorker.Batch.html#open/2","title":"FaktoryWorker.Batch.open/2","type":"function"},{"doc":"Gets the status of a batch\n\nReturns a map representing the status","ref":"FaktoryWorker.Batch.html#status/2","title":"FaktoryWorker.Batch.status/2","type":"function"},{"doc":"","ref":"FaktoryWorker.Batch.html#t:bid/0","title":"FaktoryWorker.Batch.bid/0","type":"type"},{"doc":"The `FaktoryWorker.Job` module is used to perform jobs in the background by sending to and fetching from Faktory.\n\nTo build a worker you must `use` the job module within a module in your application.\n\n```elixir\ndefmodule MyApp.SomeWorker do\n  use FaktoryWorker.Job\nend\n```\n\nThis will bring in all of the functionality required to perform jobs via Faktory. The `MyApp.SomeWorker` will now\nhave a `perform_async/2` function available for sending jobs to Faktory. Before this function can be called, a\n`perform` function must be defined. This function must accept the same number of arguments that are\nbeing sent to Faktory via the `perform_async/2` function.\n\n```elixir\ndefmodule MyApp.SomeWorker do\n  use FaktoryWorker.Job\n\n  def perform(job_arg) do\n    do_some_work(job_arg)\n  end\nend\n```\n\nWith this in place it is now possible to send work to Faktory and the `MyApp.SomeWorker` will fetch the job and call\nthe `perform/1` function with the same job arguments that we sent.\n\n```elixir\n> MyApp.SomeWorker.perform_async(\"job arg\")\n:ok\n```\n\nIt is also possible to send multiple arguments for a single job by passing in a list of values to the `perform_async/2`\nfunction.\n\n```elixir\n> MyApp.SomeWorker.perform_async([\"job arg1\", \"job arg2\"])\n:ok\n```\n\nIn order for the job to be performed correctly, a `perform/2` function needs to be defined within the `MyApp.SomeWorker`\nmodule.\n\n```elixir\ndefmodule MyApp.SomeWorker do\n  use FaktoryWorker.Job\n\n  def perform(job_arg1, job_arg2) do\n    do_some_work(job_arg1, job_arg2)\n  end\nend\n```\n\nWhen defining `perform` functions, they must always accept one argument for each item in the list of values passed into\n`perform_async/2`.","ref":"FaktoryWorker.Job.html","title":"FaktoryWorker.Job","type":"module"},{"doc":"Previous version used Broadway to send jobs and `:skip_pipeline` parameter was used to do it synchronously.\n`:skip_pipeline` is not supported anymore.\nSince Batch operations is a feature of Faktory Enterprise this library now sends any single job synchronously\nand makes HTTP call to faktory server (see `FaktoryWorker.Batch`).","ref":"FaktoryWorker.Job.html#module-synchronous-job-pushing","title":"Synchronous job pushing - FaktoryWorker.Job","type":"module"},{"doc":"A list of options can be specified when using the the `FaktoryWorker.Job` module. These options will be used when sending\njobs to faktory and will apply to all jobs sent with the `perform_async/2` function.\n\nFor a full list of configuration options see the [Worker Configuration](configuration.html#worker-configuration) documentation.","ref":"FaktoryWorker.Job.html#module-worker-configuration","title":"Worker Configuration - FaktoryWorker.Job","type":"module"},{"doc":"The `perform_async/2` function accepts a keyword list as its second argument. This list has the same options\navailable that the `FaktoryWorker.Job` module accepts. Any options passed into this function override the options\nthat have been set on the worker module.\n\nFor a full list of configuration options see the [Worker Configuration](configuration.html#worker-configuration) documentation.","ref":"FaktoryWorker.Job.html#module-overriding-worker-configuration","title":"Overriding Worker Configuration - FaktoryWorker.Job","type":"module"},{"doc":"Faktory expects all values to be serialized in JSON format. FaktoryWorker uses `Jason` for serialization. This\nmeans only values that implement the `Jason.Encoder` protocol are valid when calling the `perform_async/2` function.","ref":"FaktoryWorker.Job.html#module-data-serialization","title":"Data Serialization - FaktoryWorker.Job","type":"module"},{"doc":"When started with sandbox mode enabled, `FaktoryWorker` won't connect\nto a running Faktory instance or perform queued jobs, but will instead\nrecord all jobs enqueued locally.\n\nIn general, this mode should only be used during testing. See\n[Sandbox Testing](sandbox-testing.html) to read more.","ref":"FaktoryWorker.Sandbox.html","title":"FaktoryWorker.Sandbox","type":"module"},{"doc":"Returns true if sandbox mode is enabled.","ref":"FaktoryWorker.Sandbox.html#active?/0","title":"FaktoryWorker.Sandbox.active?/0","type":"function"},{"doc":"Returns a list of all locally enqueued jobs across all job modules, in\nthe order that they were enqueued. If you only want to retrieve jobs\nenqueued for a specific job module, use `all_jobs/1` instead.","ref":"FaktoryWorker.Sandbox.html#all_jobs/0","title":"FaktoryWorker.Sandbox.all_jobs/0","type":"function"},{"doc":"Returns a list of all locally enqueued jobs for the given job module,\nin the order that they were enqueued. If you want to retrieve a list of\nall jobs across all job modules, use `all_jobs/0` instead.","ref":"FaktoryWorker.Sandbox.html#all_jobs/1","title":"FaktoryWorker.Sandbox.all_jobs/1","type":"function"},{"doc":"","ref":"FaktoryWorker.Sandbox.html#batch_id/0","title":"FaktoryWorker.Sandbox.batch_id/0","type":"function"},{"doc":"","ref":"FaktoryWorker.Sandbox.html#batch_status/0","title":"FaktoryWorker.Sandbox.batch_status/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"FaktoryWorker.Sandbox.html#child_spec/1","title":"FaktoryWorker.Sandbox.child_spec/1","type":"function"},{"doc":"Clear the queue history for all job modules.","ref":"FaktoryWorker.Sandbox.html#reset/0","title":"FaktoryWorker.Sandbox.reset/0","type":"function"},{"doc":"Clear the queue history for the given job module.","ref":"FaktoryWorker.Sandbox.html#reset/1","title":"FaktoryWorker.Sandbox.reset/1","type":"function"},{"doc":"A concise description of a single call to a job module.","ref":"FaktoryWorker.Sandbox.html#t:job/0","title":"FaktoryWorker.Sandbox.job/0","type":"type"},{"doc":"Test utilities for asserting and refuting whether jobs are enqueued,\nas well as performing jobs as they will be performed when pulled off\nthe queue at runtime.\n\nSee [Sandbox Testing](sandbox-testing.html) to read more.","ref":"FaktoryWorker.Testing.html","title":"FaktoryWorker.Testing","type":"module"},{"doc":"Assert that a job was enqueued for the given job module, optionally matching\na set of argument/option filters. By default, this assertion will fail if more\nthan one matching job is found. If you want to assert that multiple matching\njobs are enqueued, you can use the `:count` option. If you want the assertion\nto pass if _at least_ one matching job is enqueued, pass `count: :any` (this\nshould generally be discouraged in favor of a specific count, to help catch\naccidental duplicate enqueues).","ref":"FaktoryWorker.Testing.html#assert_enqueued/2","title":"FaktoryWorker.Testing.assert_enqueued/2","type":"function"},{"doc":"defmodule MyApp.Test do\n      use ExUnit.Case\n\n      import FaktoryWorker.Testing\n\n      setup :reset_queues\n\n      test \"enqueues\" do\n        MyApp.Job.perform_async(\"argument\")\n\n        # assert that _any_ job was enqueued for this module\n        assert_enqueued MyApp.Job\n\n        # assert that a specific job was enqueued\n        assert_enqueued MyApp.Job, args: [\"argument\"]\n\n        # assert on multiple arguments (order matters)\n        assert_enqueued MyApp.Job, args: [\"foo\", \"bar\"]\n\n        # assert that two matching jobs were enqueued\n        assert_enqueued MyApp.Job, args: [\"foo\", \"bar\"], count: 2\n\n        # assert on serializable arguments\n        assert_enqueued MyApp.Job, args: [%MyApp.User{...}]\n\n        # assert on options (only those explicitly passed to `perform_async`)\n        assert_enqueued MyApp.Job, opts: [reserve_for: 1_500]\n\n        # assert on multiple filters\n        assert_enqueued MyApp.Job, args: [\"foo\", \"bar\"], opts: [custom: %{}]\n      end\n    end","ref":"FaktoryWorker.Testing.html#assert_enqueued/2-examples","title":"Examples - FaktoryWorker.Testing.assert_enqueued/2","type":"function"},{"doc":"","ref":"FaktoryWorker.Testing.html#describe_jobs/1","title":"FaktoryWorker.Testing.describe_jobs/1","type":"function"},{"doc":"Perform the job in the same way that it will be performed when pulled\nfrom the queue at runtime. Specifically, this means:\n\n- work will be done in a separate process, under a `Task.Supervisor`\n- job arguments will go through serialization/deserialization\n\nBlocks until the job has completed, returning the result of executing\nthe `perform` function. If the job raises or exits, the return value\nwill be `{:error, \"raise or exit reason\"}`. If the job times out\n(execution time exceeds the job's `:reserve_for` duration), `{:error, :timeout}`\nwill be returned instead of the normal result.\n\nNote that `ExUnit.CaptureLog`, `ExUnit.CaptureIO`, and friends will\n**not** work since the job process is not linked to the caller.","ref":"FaktoryWorker.Testing.html#perform_job/3","title":"FaktoryWorker.Testing.perform_job/3","type":"macro"},{"doc":"# arguments are optional for 0-arity jobs\n    perform_job(MyJob)\n\n    # single arguments may be passed directly\n    perform_job(MyJob, 123)\n\n    # multiple arguments in a list\n    perform_job(MyJob, [\"foo\", \"bar\"])\n\n    # you can pass options as well\n    perform_job(MyJob, [\"arg1\"], reserve_for: 1_500)\n\n    # if you need to pass options to a 0-arity job, you must\n    # pass an empty list for arguments\n    perform_job(MyJob, [], reserve_for: 1_500)","ref":"FaktoryWorker.Testing.html#perform_job/3-examples","title":"Examples - FaktoryWorker.Testing.perform_job/3","type":"macro"},{"doc":"Assert that a job was **not** enqueued for the given job module, optionally\nspecifying a set of argument/option filters.","ref":"FaktoryWorker.Testing.html#refute_enqueued/2","title":"FaktoryWorker.Testing.refute_enqueued/2","type":"function"},{"doc":"defmodule MyApp.Test do\n      use ExUnit.Case\n\n      import FaktoryWorker.Testing\n\n      setup :reset_queues\n\n      test \"doesn't enqueue\" do\n        # refute that _any_ job was enqueued for this module\n        refute_enqueued MyApp.Job\n\n        # refute that a specific job was enqueued\n        refute_enqueued MyApp.Job, args: [\"argument\"]\n\n        # refute on multiple arguments (order matters)\n        refute_enqueued MyApp.Job, args: [\"foo\", \"bar\"]\n\n        # refute on serializable arguments\n        refute_enqueued MyApp.Job, args: [%MyApp.User{...}]\n\n        # refute on options (only those explicitly passed to `perform_async`)\n        refute_enqueued MyApp.Job, opts: [reserve_for: 1_500]\n\n        # refute on multiple filters\n        refute_enqueued MyApp.Job, args: [\"foo\", \"bar\"], opts: [custom: %{}]\n      end\n    end","ref":"FaktoryWorker.Testing.html#refute_enqueued/2-examples","title":"Examples - FaktoryWorker.Testing.refute_enqueued/2","type":"function"},{"doc":"Resets the queue history for all job modules.\n\nGenerally, this should be called in the `setup` block for any tests that\nuse `assert_enqueued/2` or `refute_enqueued/2`, to ensure that test cases\ndon't pollute each other.\n\nThis will reset the history for _all_ job modules; if you need finer-grained\ncontrol over which job modules are reset (generally you shouldn't), use\n`FaktoryWorker.Sandbox.reset/1` instead.\n\nIf you don't need to perform any other setup, you can use the atom shorthand\n\n    setup :reset_queues","ref":"FaktoryWorker.Testing.html#reset_queues/1","title":"FaktoryWorker.Testing.reset_queues/1","type":"function"},{"doc":"# Faktory Worker\n\nA worker client for [Faktory](https://github.com/contribsys/faktory).","ref":"faktory-worker.html","title":"Faktory Worker","type":"extras"},{"doc":"Faktory Worker documentation is available at [https://hexdocs.pm/faktory_worker](https://hexdocs.pm/faktory_worker).","ref":"faktory-worker.html#documentation","title":"Documentation - Faktory Worker","type":"extras"},{"doc":"To get started with Faktory Worker first add the dependency to your `mix.exs` file.\n\n```elixir\ndefp deps do\n  [\n    {:faktory_worker, \"~> 1.9.6\"}\n  ]\nend\n```\n\nFaktory Worker can then be configured to start as part of your application by adding it to the `Application.start/2` function.\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_, _) do\n    children = [\n      FaktoryWorker\n    ]\n\n    Supervisor.start_link(children,\n      strategy: :one_for_one,\n      name: MyApp.Supervisor\n    )\n  end\nend\n```\n\nMake sure you have configured your application to start in your `mix.exs` file.\n\n```elixir\ndef application do\n  [\n    mod: {MyApp.Application, []}\n  ]\nend\n```\n\nThis will start Faktory Worker using the default configuration. For details on how to configure Faktory Worker\nsee the [FaktoryWorker](https://hexdocs.pm/faktory_worker/faktory-worker.html#content) documentation.\n\nNext you can create a worker to handle job processing.\n\n```elixir\ndefmodule MyApp.SomeWorker do\n  use FaktoryWorker.Job\n\n  def perform(job_data) do\n    do_some_work(job_data)\n  end\nend\n```\n\nNow you can start sending jobs to faktory and they will be automatically picked up by the worker.\n\n```elixir\n:ok = MyApp.SomeWorker.perform_async(\"hey there!\")\n```\n##","ref":"faktory-worker.html#getting-up-and-running","title":"Getting up and running - Faktory Worker","type":"extras"},{"doc":"```elixir\n{:ok, job_meta} = MyApp.SomeWorker.perform_async(\"hey there!\")\n```","ref":"faktory-worker.html#important-since-version-1-6-1-broadway-was-removed-from-dependencies-and-response-from-perfomr_async-is-tuple-ok-job_meta-with-meta-information-about-scheduled-job","title":"Important! Since version 1.6.1 Broadway was removed from dependencies and response from perfomr_async is tuple {:ok, job_meta} with meta information about scheduled job. - Faktory Worker","type":"extras"},{"doc":"It's possible to send more than one argument to Faktory by passing a list of data to `perform_async/1`. Picking up from the example in the [Getting up and running](#getting-up-and-running) section we can modify our worker by adding another `perform` function with an arity that matches the number of job arguments we expect to send to Faktory.\n\n```elixir\ndefmodule MyApp.SomeWorker do\n  use FaktoryWorker.Job\n\n  ...\n\n  def perform(arg1, arg2) do\n    do_some_work(arg1, arg2)\n  end\nend\n```\n\nWith this new function in place you can now send multiple job arguments to Faktory.\n\n```elixir\n:ok = MyApp.SomeWorker.perform_async([\"arg 1\", \"arg 2\"])\n```","ref":"faktory-worker.html#sending-multiple-job-arguments","title":"Sending multiple job arguments - Faktory Worker","type":"extras"},{"doc":"The full list of configuration options are available in the [Configuration](https://hexdocs.pm/faktory_worker/configuration.html#content) documentation.","ref":"faktory-worker.html#configuration","title":"Configuration - Faktory Worker","type":"extras"},{"doc":"By default Faktory Worker will not output any log messages but instead emit events using the [Telemetry](https://github.com/beam-telemetry/telemetry) library.\n\nTo enable the built in logging you will need to attach the default Telemetry handler provided by FaktoryWorker. The ideal place to do this is in your `Application.start/2` callback.\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_, _) do\n    FaktoryWorker.attach_default_telemetry_handler()\n\n    ...\n  end\nend\n```\n\nWith this in place Faktory Worker will now output log messages for each of the events emitted.\n\nFor a full list of Faktory Worker events or for details on handling these events see the [Logging](https://hexdocs.pm/faktory_worker/logging.html#content) documentation.","ref":"faktory-worker.html#logging","title":"Logging - Faktory Worker","type":"extras"},{"doc":"We always appreciate contributions whether they are testing, reporting issues, feedback or submitting PRs. If you would like to work on Faktory Worker please follow the [Developing](#developing) section for details on how to get setup for developing and running the test suite.","ref":"faktory-worker.html#contributing","title":"Contributing - Faktory Worker","type":"extras"},{"doc":"Faktory Worker includes a docker compose file that provisions all of the Faktory instances required to run the test suite.\n\nIf you have docker compose installed you can run the `up` command from the Faktory Worker directory to start everything required.\n\n```sh\n$ docker-compose up -d\nCreating faktory_worker_test          ... done\nCreating faktory_worker_test_tls      ... done\nCreating faktory_worker_password_test ... done\n```\n\n\nFaktory have free open-source solution and enterprise edition.\n\nIf you don't have enterprise license then tests will fail on enterprise features (batching operations etc). In this case you can exclude them by tag `:enterprise`\n```sh\n$ mix test --exclude enterprise\n```\n\nIf you are enterprise user all tests should pass\n```sh\n$ mix test\n```","ref":"faktory-worker.html#developing","title":"Developing - Faktory Worker","type":"extras"},{"doc":"# Faktory Worker Configuration","ref":"configuration.html","title":"Configuration","type":"extras"},{"doc":"Faktory Worker offers a range configuration options that are passed into the `FaktoryWorker` module directly when defining it as part of a supervision tree. All of these options are optional and have default values defined.\n\nConfiguration can be specified in the supervision tree using a two element tuple.\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_, _) do\n    children = [\n      {FaktoryWorker, [name: :faktory_test]}\n    ]\n\n    ...\n  end\nend\n```\n\nBelow is the full structure of available options and a definition for each one.\n\n```elixir\n[\n  name: atom(),\n  connection: [\n    host: String.t(),\n    port: pos_integer(),\n    password: String.t(),\n    use_tls: boolean()\n  ],\n  pool: [\n    size: pos_integer(),\n    buffer_size: pos_integer() | :infinity\n  ],\n  worker_pool: [\n    size: pos_integer(),\n    disable_fetch: boolean(),\n    queues: [\n      list(String.t() | {String.t(), keyword()})\n    ]\n  ]\n]\n```\n\n#","ref":"configuration.html#available-configuration","title":"Available Configuration - Configuration","type":"extras"},{"doc":"- `name` (default: `FaktoryWorker`) - The name to use for this instance. This is useful if you want to run more than one instance of Faktory Worker. When using this make sure the workers are configured to use the correct instance (see [Worker Configuration](#worker-configuration) below).\n\n- `connection` - A list of options to configure the connection to Faktory.\n\n  - `host` (default: `\"localhost\"`) - The host name used to connect to Faktory.\n  - `port` (default: `7419`) - The tcp port used to connect to Faktory.\n  - `password` (default: `nil`) - The password used to authenticate with Faktory.\n  - `use_tls` (default: `false`) - A boolean flag that indicates whether Faktory Worker should connect using a TLS connection.\n\n- `pool` - A list of options to configure the pool of Faktory connections used for sending jobs to Faktory.\n\n  - `size` (default: `10`) - The number of connections to open to Faktory.\n  - `buffer_size` (default: `:infinity`) - The number of jobs that can be stored in memory whilst waiting for an available connection from the pool. Note that this defaults to `:infinity` to prevent losing jobs during high load scenarios. This could potentially lead to increased memory consumption if there is low throughput to Faktory.\n\n- `worker_pool` - A list of options to configure the Faktory workers.\n  - `size` (default: `10`) - The number of worker connections to open to Faktory.\n  - `disable_fetch` (default: `false`) - A boolean flag used to disable fetching on each of the workers. This is useful if you want to prevent the workers from fetching new jobs without having to remove the queues config.\n  - `queues` (default: `[\"default\"]`) - A list of queues that the workers should fetch jobs from. See the [Configuring Queues](#configuring-queues) section below for more details.","ref":"configuration.html#option-definitions","title":"Option Definitions - Configuration","type":"extras"},{"doc":"Queues are configured by specifying a list of strings or tuples.\n\n```elixir\nqueues: [\"queue_name\", {\"another_queue\", opts}]\n```\n\nThe queue name must match the name of the queue in Faktory. Using the tuple definition allows a list of options to be specified when fetching jobs from that queue.\n\nThe available options are.\n\n```elixir\n[\n  max_concurrency: pos_integer() | :infinity\n]\n```\n\n#","ref":"configuration.html#configuring-queues","title":"Configuring Queues - Configuration","type":"extras"},{"doc":"- `max_concurrency` (default: `:infinity`) - The maximum number of jobs that should be fetched and processed from this queue in parallel.","ref":"configuration.html#option-definitions","title":"Option Definitions - Configuration","type":"extras"},{"doc":"Workers can be configured with queue specific options that are used by Faktory Worker when sending jobs to Faktory. All of these options are optional and have default values.\n\nOptions are defined when using `FaktoryWorker.Job`.\n\n```elixir\nuse FaktoryWorker.Job, opts\n```\n\nBelow is the full structure of available options and a definition for each one.\n\n```elixir\n[\n  queue: String.t(),\n  jobtype: String.t(),\n  at: DateTime.t(),\n  retry: pos_integer(),\n  reserve_for: pos_integer(),\n  custom: map(),\n  faktory_name: atom()\n]\n```\n\n#","ref":"configuration.html#worker-configuration","title":"Worker Configuration - Configuration","type":"extras"},{"doc":"- `queue` (default: `\"default\"`) - The name of the queue in Faktory to send jobs to.\n- `jobtype` (default: `__MODULE__`) - The type of job to send. This is used to determine how to process a job when fetched from Faktory.\n- `at` (relies on Faktory default) - The date and time Faktory should run the job.\n- `retry` (relies on Faktory default) - The number of times the job should retry if it fails.\n- `reserve_for` (relies on Faktory default) -The number of seconds the job is reserved for in Faktory before it is considered failed.\n- `custom` (relies on Faktory default) - A map of values to be included with the job when it is sent to Faktory.\n- `faktory_name` (default: `FaktoryWorker`) - The name of the Faktory instance to use when sending the job to Faktory.","ref":"configuration.html#option-definitions","title":"Option Definitions - Configuration","type":"extras"},{"doc":"# Faktory Worker Logging\n\nFaktory Worker uses the [Telemetry](https://github.com/beam-telemetry/telemetry) library to emit events that can be handled by either Faktory Worker or the consuming application. This provides the flexibility for the consumer of Faktory Worker to choose how logging should behave.\n\nBy default Faktory Worker will not output any log messages. Logging needs to be explicitly enabled by calling the `FaktoryWorker.attach_default_telemetry_handler/0` function.\n\nEvents can also be handled in the consuming application by manually attaching to Telemetry specifying the list of Faktory Worker events to handle. For instructions on creating a custom event handler see the Telemetry [README](https://github.com/beam-telemetry/telemetry/blob/master/README.md).\n\nAll Faktory Worker events are prefixed with `:faktory_worker` and have a single event name. For example the full event name for the `push` event is `[:faktory_worker, :push]`.\n\nThe measurements data included for all events will be a map containing a `status` field that will indicate the outcome of the event being emitted. The available outcomes are listed individually below for each type of event.","ref":"logging.html","title":"Logging","type":"extras"},{"doc":"Event Name: `[:faktory_worker, :push]`.\n\nThe push event is emitted when a job has been sent to Faktory. The `status` of this event indicates whether the job was sent successfully.\n\nThe available status outcomes are.\n- `:ok` - The job was successfully sent to Faktory.\n- `{:error, :not_unique}` - The job was not sent to Faktory because another job with the same uniqueness already exists.\n\nThe `metadata` supplied with this event is a map containing the Faktory job arguments.","ref":"logging.html#push-event","title":"Push Event - Logging","type":"extras"},{"doc":"Event Name: `[:faktory_worker, :fetch]`.\n\nThe fetch event is emitted when attempting to fetch a job from Faktory. The `status` of this event indicates whether the fetch request was sent successfully.\n\nThe available status outcomes are.\n- `{:error, reason}` - The request to fetch from Faktory failed. The `reason` will be any valid `term` indicating the reason the fetch failed.\n\nThe `metadata` supplied with this event is a map containing the following fields.\n- `wid` - A worker identifier that can be used to identify which worker emitted this event.","ref":"logging.html#fetch-event","title":"Fetch Event - Logging","type":"extras"},{"doc":"Event Name: `[:faktory_worker, :ack]`.\n\nThe ack event is emitted when reporting the success or failure of a processed job to Faktory. The `status` of this event indicates the outcome of processing the job.\n\nThe available status outcomes are.\n- `:ok` - The job completed successfully and has been reported to Faktory.\n- `:error` - The job failed to complete and has been reported to Faktory with details about the failure.\n\nThe `metadata` supplied with this event is a map containing the Faktory job arguments.","ref":"logging.html#ack-event","title":"Ack Event - Logging","type":"extras"},{"doc":"Event Name: `[:faktory_worker, :failed_ack]`.\n\nThe failed ack event is emitted when Faktory Worker was unable to report the outcome of processing a job to Faktory. The `status` of this event indicates the outcome of the processed job that failed to reach Faktory.\n\nThe available status outcomes are.\n- `:ok` - The job completed successfully.\n- `:error` - The job failed to complete.\n\nThe `metadata` supplied with this event is a map containing the Faktory job arguments.","ref":"logging.html#failed-ack-event","title":"Failed Ack Event - Logging","type":"extras"},{"doc":"Event Name: `[:faktory_worker, :beat]`.\n\nThe beat event is emitted every time a worker has attempted to send a heartbeat message to Faktory. The `status` of this event indicates whether the heartbeat message was successfully received by Faktory.\n\nThe available status outcomes are.\n- `:ok` - The heartbeat message was successfully received by Faktory.\n- `:error` - The heartbeat message failed to reach Faktory.\n\nThe `metadata` supplied with this event is a map containing the following fields.\n- `wid` - A worker identifier that can be used to identify which worker emitted this event.\n- `prev_status` - The `status` that was emitted on the previous `beat` event. This is useful for tracking when the status has changed between heartbeats.","ref":"logging.html#beat-event","title":"Beat Event - Logging","type":"extras"},{"doc":"Event Name: `[:faktory_worker, :job_timeout]`.\n\nThe job timeout event is emitted 20 seconds prior to a job's reservation deadline. This 20-second window allows the job to be terminated before Faktory can expire and retry the job on the server.\n\nThe `status` status provided for this event will always be `{:error, :job_timeout}`.\n\nThe `metadata` supplied with this event is a map containing the job's ID, arguments, and type.","ref":"logging.html#job-timeout-event","title":"Job Timeout Event - Logging","type":"extras"},{"doc":"# Sandbox Testing\n\nWhen testing your application, you may not want to actually push/pull jobs to\nFaktory, but instead rely on asserting that jobs _would be_ enqueued as you\nexpect. Enabling sandbox mode will accomodate this; when sandbox mode is active,\nthe following changes are made:\n\n- no connection will be made to Faktory (you don't even need a Faktory server running!)\n- jobs enqueued (with `perform_async/2`) will **never** be run\n- jobs enqueued will be recorded, and can be viewed using `FaktoryWorker.Sandbox`\n\nTo enable sandbox mode, set `:sandbox` to `true`:\n\n```elixir\n# config/test.exs\n\nconfig :my_app, FaktoryWorker, sandbox: true\n```\n\n```elixir\n# lib/my_app/application.ex\n\ndefmodule MyApp.Application do\n  @moduledoc false\n\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {FaktoryWorker, Application.get_env(:my_app, FaktoryWorker)}\n    ]\n\n    opts = [name: MyApp.Supervisor, strategy: :one_for_one]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nYou can verify that sandbox mode is enabled with `FaktoryWorker.Sandbox.active?/0`.","ref":"sandbox-testing.html","title":"Sandbox Testing","type":"extras"},{"doc":"To aid in tests that rely on jobs being enqueued, as well as tests for job modules\nthemselves, you can use the helpers in `FaktoryWorker.Testing`. Below is a simple\ntest for a module that enqueues a job:\n\n```elixir\ndefmodule MyApp.QueueTest do\n  use ExUnit.Case\n\n  import FaktoryWorker.Testing\n\n  setup :reset_queues\n\n  test \"perform/1\" do\n    MyApp.Job.perform_async(\"hello, world!\")\n    assert_enqueued MyApp.Job, args: [\"hello, world!\"]\n    refute_enqueued MyApp.Job, args: [\"goodbye!\"]\n  end\n\n  test \"multiple enqueues\" do\n    for _ <- 1..10 do\n      MyApp.Job.perform_async(\"brrrrr\")\n    end\n\n    assert_enqueued MyApp.Job, count: 10\n  end\n\n  test \"perform/2\" do\n    MyApp.Job.perform_async([\"foo\", \"bar\"])\n    assert_enqueued MyApp.Job, args: [\"foo\", \"bar\"]\n  end\n\n  test \"with opts\" do\n    MyApp.Job.perform_async(\"howdy\", reserve_for: 1_500)\n    assert_enqueued MyApp.Job, opts: [reserve_for: 1_500]\n\n    # these would work too!\n    # assert_enqueued MyApp.Job, args: [\"howdy\"]\n    # assert_enqueued MyApp.Job, args: [\"howdy\"], opts: [reserve_for: 1_500]\n  end\nend\n```\n\nHere's a simple example test for a job module itself:\n\n```elixir\ndefmodule MyApp.JobTest do\n  use ExUnit.Case\n\n  import FaktoryWorker.Testing\n\n  test \"handles no arguments\" do\n    assert perform_job(MyApp.Job) == :ok\n  end\n\n  test \"handles strings\" do\n    assert perform_job(MyApp.Job, \"foo\") == :ok\n  end\n\n  test \"handles maps\" do\n    assert perform_job(MyApp.Job, %{id: 1}) == {:ok, \"cool, a map!\"}\n  end\n\n  test \"handles structs\" do\n    # structs are serialized and passed to `perform` as bare maps\n    assert perform_job(MyApp.Job, %MyApp.User{id: 1}) == {:ok, \"cool, a map!\"}\n  end\n\n  test \"doesn't handle numbers\" do\n    assert perform_job(MyApp.Job, 1_234) == {:error, \"boo, no numbers!\"}\n  end\n\n  test \"gracefully handles raises\" do\n    assert perform_job(MyApp.Job, \"I'll cause a raise!\") == {:error, \"raise reason\"}\n  end\nend\n```\n\nWhen you can, prefer to use `perform_job/3` instead of calling `perform` directly, as\nit will ensure that the code under test is run in the same way that it will eventually\nbe at runtime.","ref":"sandbox-testing.html#writing-tests","title":"Writing Tests - Sandbox Testing","type":"extras"},{"doc":"# Mutate API\n\nFrom the [Faktory documentation](https://github.com/contribsys/faktory/wiki/Mutate-API):\n\n> It's inevitable: you will have cases where you enqueue bad data or have jobs\n> which need pruning, migration or some other rare use case. For this, Faktory\n> provides low-level data management APIs so you can script certain repairs or\n> migrations.\n\n> Please be warned: MUTATE commands can be slow and/or resource intensive.\n> They should not be used as part of your application logic.","ref":"mutate.html","title":"Mutate API","type":"extras"},{"doc":"Support for the Mutate API is currently very basic. At some point in the future,\nthis may be enhanced with a first-class Elixir API, but for now calls must be\nmade manually:\n\n```elixir\nargs = %{\n  cmd: \"kill\",\n  target: \"default\",\n  filter: %{jobtype: \"MyApp.Job\"}\n}\n\nFaktoryWorker.send_command({:mutate, args})\n```\n\nSee [here](https://github.com/contribsys/faktory/wiki/Mutate-API#mutate) for the\nfull `MUTATE` argument reference.","ref":"mutate.html#usage-experimental","title":"Usage (Experimental) - Mutate API","type":"extras"}]}